From: Dev Jain <dev.jain@arm.com>
To: Lorenzo Stoakes <lorenzo.stoakes@oracle.com>
Cc: akpm@linux-foundation.org, Liam.Howlett@oracle.com,
vbabka@suse.cz, jannh@google.com, pfalcato@suse.de,
linux-mm@kvack.org, linux-kernel@vger.kernel.org,
david@redhat.com, peterx@redhat.com, ryan.roberts@arm.com,
mingo@kernel.org, libang.li@antgroup.com, maobibo@loongson.cn,
zhengqi.arch@bytedance.com, baohua@kernel.org,
anshuman.khandual@arm.com, willy@infradead.org,
ioworker0@gmail.com, yang@os.amperecomputing.com,
baolin.wang@linux.alibaba.com, ziy@nvidia.com, hughd@google.com
Subject: Re: [PATCH v4 0/2] Optimize mremap() for large folios
Date: Tue, 10 Jun 2025 18:03:09 +0530 [thread overview]
Message-ID: <c05975a5-9a4d-4b23-9768-18b9534333b6@arm.com> (raw)
In-Reply-To: <7cf96262-85b4-4a7f-9ca3-de627ee13a0d@lucifer.local>
On 10/06/25 5:41 pm, Lorenzo Stoakes wrote:
> On Tue, Jun 10, 2025 at 09:20:41AM +0530, Dev Jain wrote:
>> Currently move_ptes() iterates through ptes one by one. If the underlying
>> folio mapped by the ptes is large, we can process those ptes in a batch
>> using folio_pte_batch(), thus clearing and setting the PTEs in one go.
>> For arm64 specifically, this results in a 16x reduction in the number of
>> ptep_get() calls (since on a contig block, ptep_get() on arm64 will iterate
>> through all 16 entries to collect a/d bits), and we also elide extra TLBIs
>> through get_and_clear_full_ptes, replacing ptep_get_and_clear.
> Thanks this is good!
>
>> Mapping 1M of memory with 64K folios, memsetting it, remapping it to
>> src + 1M, and munmapping it 10,000 times, the average execution time
>> reduces from 1.9 to 1.2 seconds, giving a 37% performance optimization,
>> on Apple M3 (arm64). No regression is observed for small folios.
> Hmm, I thought people were struggling to get M3 to work with Asahi? :) or is
> this in a mac-based vm? I've not paid attention to recent developments.
I meant a Linux VM on Mac.
>
>> The patchset is based on mm-unstable (6ebffe676fcf).
>>
>> Test program for reference:
>>
>> #define _GNU_SOURCE
>> #include <stdio.h>
>> #include <stdlib.h>
>> #include <unistd.h>
>> #include <sys/mman.h>
>> #include <string.h>
>> #include <errno.h>
>>
>> #define SIZE (1UL << 20) // 1M
>>
>> int main(void) {
>> void *new_addr, *addr;
>>
>> for (int i = 0; i < 10000; ++i) {
>> addr = mmap((void *)(1UL << 30), SIZE, PROT_READ | PROT_WRITE,
>> MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
>> if (addr == MAP_FAILED) {
>> perror("mmap");
>> return 1;
>> }
>> memset(addr, 0xAA, SIZE);
>>
>> new_addr = mremap(addr, SIZE, SIZE, MREMAP_MAYMOVE | MREMAP_FIXED, addr + SIZE);
>> if (new_addr != (addr + SIZE)) {
>> perror("mremap");
>> return 1;
>> }
>> munmap(new_addr, SIZE);
>> }
>>
>> }
>>
> Thanks for including! Very useful.
>
>> v3->v4:
>> - Remove comment above mremap_folio_pte_batch, improve patch description
>> differentiating between folio splitting and pagetable splitting
>> v2->v3:
>> - Refactor mremap_folio_pte_batch, drop maybe_contiguous_pte_pfns, fix
>> indentation (Lorenzo), fix cover letter description (512K -> 1M)
> It's nitty but these seem to be getting more and more abbreviated :) not a
> massive big deal however ;)
>
>> v1->v2:
>> - Expand patch descriptions, move pte declarations to a new line,
>> reduce indentation in patch 2 by introducing mremap_folio_pte_batch(),
>> fix loop iteration (Lorenzo)
>> - Merge patch 2 and 3 (Anshuman, Lorenzo)
>> - Fix maybe_contiguous_pte_pfns (Willy)
>>
>> Dev Jain (2):
>> mm: Call pointers to ptes as ptep
>> mm: Optimize mremap() by PTE batching
>>
>> mm/mremap.c | 58 ++++++++++++++++++++++++++++++++++++++---------------
>> 1 file changed, 42 insertions(+), 16 deletions(-)
>>
>> --
>> 2.30.2
>>
prev parent reply other threads:[~2025-06-10 12:33 UTC|newest]
Thread overview: 24+ messages / expand[flat|nested] mbox.gz Atom feed top
2025-06-10 3:50 Dev Jain
2025-06-10 3:50 ` [PATCH v4 1/2] mm: Call pointers to ptes as ptep Dev Jain
2025-06-11 13:23 ` David Hildenbrand
2025-06-11 13:25 ` Dev Jain
2025-06-11 13:29 ` Lorenzo Stoakes
2025-06-11 13:31 ` David Hildenbrand
2025-06-12 12:05 ` Pedro Falcato
2025-06-10 3:50 ` [PATCH v4 2/2] mm: Optimize mremap() by PTE batching Dev Jain
2025-06-10 7:03 ` Barry Song
2025-06-10 7:44 ` Dev Jain
2025-06-10 8:11 ` Barry Song
2025-06-16 21:27 ` Ryan Roberts
2025-06-10 8:37 ` Barry Song
2025-06-10 13:18 ` Lorenzo Stoakes
2025-06-11 14:00 ` David Hildenbrand
2025-06-13 4:24 ` Dev Jain
2025-06-17 8:02 ` David Hildenbrand
2025-06-13 12:32 ` Lorenzo Stoakes
2025-06-16 16:13 ` David Hildenbrand
2025-06-12 12:13 ` Pedro Falcato
2025-10-27 21:40 ` David Hildenbrand
2025-10-28 5:32 ` Dev Jain
2025-06-10 12:11 ` [PATCH v4 0/2] Optimize mremap() for large folios Lorenzo Stoakes
2025-06-10 12:33 ` Dev Jain [this message]
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=c05975a5-9a4d-4b23-9768-18b9534333b6@arm.com \
--to=dev.jain@arm.com \
--cc=Liam.Howlett@oracle.com \
--cc=akpm@linux-foundation.org \
--cc=anshuman.khandual@arm.com \
--cc=baohua@kernel.org \
--cc=baolin.wang@linux.alibaba.com \
--cc=david@redhat.com \
--cc=hughd@google.com \
--cc=ioworker0@gmail.com \
--cc=jannh@google.com \
--cc=libang.li@antgroup.com \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-mm@kvack.org \
--cc=lorenzo.stoakes@oracle.com \
--cc=maobibo@loongson.cn \
--cc=mingo@kernel.org \
--cc=peterx@redhat.com \
--cc=pfalcato@suse.de \
--cc=ryan.roberts@arm.com \
--cc=vbabka@suse.cz \
--cc=willy@infradead.org \
--cc=yang@os.amperecomputing.com \
--cc=zhengqi.arch@bytedance.com \
--cc=ziy@nvidia.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®